/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.components;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.ImageIcon;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.DocumentFilter;
import javax.swing.text.InternationalFormatter;

import org.jdesktop.layout.GroupLayout;
import org.jdesktop.layout.LayoutStyle;
import org.netbeans.lib.awtextra.AbsoluteConstraints;
import org.netbeans.lib.awtextra.AbsoluteLayout;

import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.printing.FrameSettings;
import com.teamdev.jxbrowser.printing.PageRange;
import com.teamdev.jxbrowser.printing.PrintProgressListener;
import com.teamdev.jxbrowser.printing.PrintSettings;
import com.teamdev.jxbrowser.printing.PrintToFileSettings;
import com.teamdev.jxbrowser.printing.WebBrowserPrinting;
import com.teamdev.jxbrowser.printing.FrameSettings.HowToEnableFrameUI;
import com.teamdev.jxbrowser.printing.FrameSettings.PrintFrameTypeUsage;
import com.teamdev.jxbrowser.printing.PageRange.RangeType;
import com.teamdev.jxbrowser.printing.PrintSettings.PageOrientation;
import com.teamdev.xpcom.Xpcom;

@SuppressWarnings("serial")
public final class PrintDialog extends javax.swing.JDialog
{

    private JComboBox printerComboBox;

    private JCheckBox printToFileChBox;

    private JRadioButton allPagesRadioButton;

    private JRadioButton selectedPagesRadioButton;

    private NumberTextFild firstPage;

    private NumberTextFild lastPage;

    private JSpinner numberOfCopiesSpinner;

    private JCheckBox collateCheckBox;

    private JLabel collateImg;

    private JRadioButton portraitOrRButton;

    private JRadioButton landscapeOrRButton;

    private ScaleTextFild scaleTextFild;

    private JCheckBox fitToPageCheckBox;

    private JCheckBox printBgroundChBox;

    private JRadioButton asIsRButton;

    private JRadioButton onlySelectedRButton;

    private JRadioButton oneFrameOnePageRButton;

    private boolean noPrinter = true;

    private WebBrowserPrinting _printingAPI;

    private final CloseKeyAdapter closeKeyAdapter = new CloseKeyAdapter();

    public PrintDialog(JFrame owner, WebBrowser browser)
    {
        super(owner, "Print");
        setModal(true);
        _printingAPI = browser.getPrinting();
        //_printingAPI.initPrintSettingsFromDefaultPrinter();
        createGUI();
        if (Xpcom.isLinux() || Xpcom.isMacOSX())
        {
            setSize(490, 530);
        }
        else
        {
            setSize(450, 540);
        }
        setResizable(false);
        setLocationRelativeTo(browser.getComponent());
        initializePrinters();

        setVisible(true);
    }

    public void createGUI()
    {

        // Printer pane
        getContentPane().add(createPrinterPane(), BorderLayout.NORTH);

        // Properties pane
        JPanel propertiesPane = new JPanel();
        propertiesPane.setBorder(new EmptyBorder(0, 15, 0, 15));
        propertiesPane.setLayout(new BorderLayout());
        JPanel gridPane = new JPanel(new GridLayout(0, 2));
        gridPane
                .setPreferredSize(new Dimension(propertiesPane.getWidth(), 110));
        gridPane.add(createPrintRangePanel());
        gridPane.add(createNumberOfCopiesPanel());
        propertiesPane.add(gridPane, BorderLayout.NORTH);

        JPanel pagePropPane = new JPanel();
        pagePropPane.setLayout(new BorderLayout());
        JPanel pageProperites = createPagePropertiesPanel();
        pageProperites.setPreferredSize(new Dimension(pagePropPane.getWidth(),
                105));
        pagePropPane.add(pageProperites, BorderLayout.NORTH);
        pagePropPane.add(createFramesPane(), BorderLayout.CENTER);
        propertiesPane.add(pagePropPane, BorderLayout.CENTER);

        getContentPane().add(propertiesPane, BorderLayout.CENTER);
        // buttons
        getContentPane().add(createConfirmButtonsPane(), BorderLayout.SOUTH);
        pack();
    }

    JPanel createPrinterPane()
    {
        JPanel printerPane = new JPanel();
        printerComboBox = new JComboBox();
        printerComboBox.addKeyListener(new CloseKeyAdapter());
        if (Xpcom.isWindows())
        {
            printerComboBox.setPreferredSize(new Dimension(350, 22));
        }
        else if (Xpcom.isLinux())
        {
            printerComboBox.setPreferredSize(new Dimension(388, 22));
        }
        else if (Xpcom.isMacOSX())
        {
            printerComboBox.setPreferredSize(new Dimension(393, 22));
        }
        printToFileChBox = new JCheckBox("Print to file");
        printToFileChBox.addKeyListener(new CloseKeyAdapter());
        printToFileChBox.addChangeListener(new ChangeListener() {

            public void stateChanged(ChangeEvent e)
            {
                    printerComboBox.setEnabled(!printToFileChBox.isSelected());
            }
            
        });

        if (!Xpcom.isLinux())
        {
            printToFileChBox.setEnabled(false);
        }

        printerPane.setBorder(javax.swing.BorderFactory.createCompoundBorder(
                javax.swing.BorderFactory.createEmptyBorder(15, 15, 15, 15),
                javax.swing.BorderFactory.createTitledBorder("Printer")));

        printerPane.setLayout(new BoxLayout(printerPane, BoxLayout.Y_AXIS));

        JLabel nameLabel = new JLabel("Name:");

        JPanel comboBoxPanel = new JPanel();
        comboBoxPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        comboBoxPanel.setPreferredSize(new Dimension(
                printerComboBox.getWidth(), 30));
        comboBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 0));
        comboBoxPanel.setBorder(new EmptyBorder(0, 0, 11, 0));
        comboBoxPanel.add(nameLabel);
        comboBoxPanel.add(printerComboBox);

        JPanel printToFileChBoxPanel = new JPanel();
        printToFileChBoxPanel.setPreferredSize(new Dimension(printerComboBox
                .getWidth(), 25));
        // comboBoxPanel.setBorder(new EmptyBorder(0,0,11,0));
        printToFileChBoxPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
        printToFileChBoxPanel.setAlignmentY(Component.TOP_ALIGNMENT);
        printToFileChBoxPanel.add(printToFileChBox);

        printerPane.add(comboBoxPanel);
        printerPane.add(printToFileChBoxPanel);

        return printerPane;
    }

    JPanel createPrintRangePanel()
    {
        JPanel printRangePanel = new JPanel();
        printRangePanel.setBorder(new CompoundBorder(
                new EmptyBorder(0, 0, 0, 6), new TitledBorder("Page range")));
        printRangePanel.setLayout(new BoxLayout(printRangePanel,
                BoxLayout.Y_AXIS));

        ButtonGroup buttonGroup = new ButtonGroup();
        allPagesRadioButton = new JRadioButton("All");
        allPagesRadioButton.addKeyListener(closeKeyAdapter);
        selectedPagesRadioButton = new JRadioButton("Pages from:");
        selectedPagesRadioButton.addKeyListener(closeKeyAdapter);
        allPagesRadioButton.setAlignmentX(Component.LEFT_ALIGNMENT);
        selectedPagesRadioButton.setAlignmentX(Component.LEFT_ALIGNMENT);

        firstPage = new NumberTextFild();
        firstPage.setText("1");
        lastPage = new NumberTextFild();
        lastPage.setText("1");
        firstPage.setEnabled(false);
        lastPage.setEnabled(false);

        selectedPagesRadioButton.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                if (selectedPagesRadioButton.isSelected())
                {
                    firstPage.setEnabled(true);
                    lastPage.setEnabled(true);
                }
                else
                {
                    firstPage.setEnabled(false);
                    lastPage.setEnabled(false);
                }
            }
        });

        firstPage.addKeyListener(new KeyListener()
        {
            public void keyPressed(KeyEvent e)
            {
            }

            public void keyReleased(KeyEvent e)
            {
                if (firstPage.getValue() >= lastPage.getValue())
                {
                    lastPage.setValue(firstPage.getValue() + 1);
                }
            }

            public void keyTyped(KeyEvent e)
            {
            }
        });

        allPagesRadioButton.setSelected(true);
        buttonGroup.add(allPagesRadioButton);
        buttonGroup.add(selectedPagesRadioButton);

        JPanel allRadioButtonPanel = new JPanel();
        allRadioButtonPanel.setPreferredSize(new Dimension(50, 19));
        allRadioButtonPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        // allRadioButtonPanel.setBorder(new EmptyBorder(0, 5, 0, 0));
        allRadioButtonPanel.setLayout(new BoxLayout(allRadioButtonPanel,
                BoxLayout.Y_AXIS));
        allRadioButtonPanel.add(allPagesRadioButton);

        printRangePanel.add(allRadioButtonPanel);
        JPanel pagesPanel = new JPanel();
        pagesPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
        pagesPanel.setPreferredSize(new Dimension(50, 30));
        pagesPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 0, 5));
        pagesPanel.add(selectedPagesRadioButton);
        pagesPanel.add(firstPage);
        pagesPanel.add(new JLabel("  to  "));
        pagesPanel.add(lastPage);
        printRangePanel.add(pagesPanel);
        return printRangePanel;
    }

    JPanel createNumberOfCopiesPanel()
    {
        JPanel numberOfCopiesPanel = new JPanel();
        numberOfCopiesPanel.setBorder(new TitledBorder("Copies"));

        collateCheckBox = new JCheckBox("Collate");
        collateCheckBox.setEnabled(false);
        collateCheckBox.addKeyListener(closeKeyAdapter);
        collateCheckBox.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                if (collateCheckBox.isSelected())
                {
                    collateImg.setIcon(new ImageIcon(getClass().getResource(
                            "res/sorted.png")));
                }
                else
                {
                    collateImg.setIcon(new ImageIcon(getClass().getResource(
                            "res/grouped.png")));
                }
            }
        });

        numberOfCopiesSpinner = new JSpinner();
        numberOfCopiesSpinner.setModel(new SpinnerNumberModel(1, 1, 999999999,
                1));
        ((JSpinner.NumberEditor) numberOfCopiesSpinner.getEditor())
                .getTextField().addKeyListener(closeKeyAdapter);
        numberOfCopiesSpinner.addChangeListener(new ChangeListener()
        {
            public void stateChanged(ChangeEvent e)
            {
                final JSpinner model = (JSpinner) e.getSource();
                Integer value = (Integer) model.getValue();
                collateCheckBox.setEnabled(value > 1);
            }

        });

        collateImg = new JLabel();
        collateImg.setIcon(new ImageIcon(getClass().getResource(
                "res/grouped.png")));
        numberOfCopiesPanel.setLayout(new AbsoluteLayout());
        numberOfCopiesPanel.add(new JLabel("Number of copies:"),
                new AbsoluteConstraints(10, 25, -1, -1));
        if (Xpcom.isLinux() || Xpcom.isMacOSX())
        {
            numberOfCopiesPanel.add(numberOfCopiesSpinner,
                    new AbsoluteConstraints(127, 22, 89, 22));
            numberOfCopiesPanel.add(collateCheckBox, new AbsoluteConstraints(7,
                    47, -1, -1));
        }
        else
        {
            numberOfCopiesPanel.add(numberOfCopiesSpinner,
                    new AbsoluteConstraints(100, 22, 83, 23));
            numberOfCopiesPanel.add(collateCheckBox, new AbsoluteConstraints(7,
                    47, -1, -1));
        }
        numberOfCopiesPanel.add(collateImg, new AbsoluteConstraints(77, 50,
                110, 41));
        return numberOfCopiesPanel;
    }

    JPanel createPagePropertiesPanel()
    {
        JPanel propPanel = new JPanel();
        propPanel.setBorder(new CompoundBorder(new EmptyBorder(12, 0, 0, 0),
                new TitledBorder("Page properties")));
        ButtonGroup btGroup = new ButtonGroup();

        JPanel portraitPanel = new JPanel();
        portraitPanel.setLayout(new BoxLayout(portraitPanel, BoxLayout.Y_AXIS));

        portraitOrRButton = new JRadioButton();
        portraitOrRButton.addKeyListener(closeKeyAdapter);
        portraitOrRButton.setIcon(new ImageIcon(PrintDialog.class
                .getResource("res/portrait.png")));
        portraitOrRButton.setSelectedIcon(new ImageIcon(PrintDialog.class
                .getResource("res/portrait_sel.png")));
        portraitOrRButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        btGroup.add(portraitOrRButton);
        portraitPanel.add(portraitOrRButton);
        JLabel portrLabel = new JLabel("Portrait");
        portrLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        portraitPanel.add(portrLabel);
        JPanel landscapePanel = new JPanel();
        landscapePanel
                .setLayout(new BoxLayout(landscapePanel, BoxLayout.Y_AXIS));
        landscapeOrRButton = new JRadioButton();
        landscapeOrRButton.addKeyListener(closeKeyAdapter);
        landscapeOrRButton.setIcon(new ImageIcon(PrintDialog.class
                .getResource("res/landscape.png")));
        landscapeOrRButton.setSelectedIcon(new ImageIcon(PrintDialog.class
                .getResource("res/landscape_sel.png")));
        landscapeOrRButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        btGroup.add(landscapeOrRButton);
        landscapePanel.add(landscapeOrRButton);
        JLabel landscapeLabel = new JLabel("Landscape");
        landscapeLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
        landscapePanel.add(landscapeLabel);
        PrintSettings printSettings = _printingAPI.getCurrentPrintSettings();
        if (printSettings.getOrientation() == PageOrientation.Portrait)
        {
            portraitOrRButton.setSelected(true);
        }
        else if (printSettings.getOrientation() == PageOrientation.Landscape)
        {
            landscapeOrRButton.setSelected(true);
        }
        else
        {
            portraitOrRButton.setSelected(true);
        }

        scaleTextFild = new ScaleTextFild();
        PrintSettings settings = _printingAPI.getCurrentPrintSettings();
        scaleTextFild.setScaleValue(settings.getScaling() * 100D);
        scaleTextFild.addKeyListener(closeKeyAdapter);

        fitToPageCheckBox = new JCheckBox("Fit to page");
        fitToPageCheckBox.addKeyListener(closeKeyAdapter);

        printBgroundChBox = new JCheckBox("Print backgrounds");
        printBgroundChBox.addKeyListener(closeKeyAdapter);

        if (printSettings.isPrintBackgroundColors()
                || printSettings.isPrintBackgroundImages())
        {
            printBgroundChBox.setSelected(true);
        }
        else
        {
            printBgroundChBox.setSelected(false);
        }

        fitToPageCheckBox.addChangeListener(new ChangeListener()
        {

            public void stateChanged(ChangeEvent e)
            {
                scaleTextFild.setEnabled(!fitToPageCheckBox.isSelected());
            }

        });

        JLabel orientation = new JLabel("Orientation:");
        JLabel scale = new JLabel(" Scale:");
        if (Xpcom.isLinux() || Xpcom.isMacOSX())
        {
            GroupLayout propPanelLayout = new GroupLayout(propPanel);
            propPanelLayout
                    .setHorizontalGroup(propPanelLayout
                            .createParallelGroup(GroupLayout.LEADING)
                            .add(
                                    propPanelLayout
                                            .createSequentialGroup()
                                            .add(
                                                    propPanelLayout
                                                            .createParallelGroup(
                                                                    GroupLayout.LEADING)
                                                            .add(
                                                                    propPanelLayout
                                                                            .createSequentialGroup()
                                                                            .add(
                                                                                    scale)
                                                                            .add(
                                                                                    5,
                                                                                    5,
                                                                                    5)
                                                                            .add(
                                                                                    scaleTextFild,
                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                    85,
                                                                                    GroupLayout.PREFERRED_SIZE)
                                                                            .add(
                                                                                    5,
                                                                                    5,
                                                                                    5)
                                                                            .add(
                                                                                    fitToPageCheckBox))
                                                            .add(
                                                                    printBgroundChBox))
                                            .add(15, 15, 15).add(orientation)
                                            .add(portraitPanel, 0, 54,
                                                    GroupLayout.PREFERRED_SIZE)
                                            .add(landscapePanel, 0,
                                                    GroupLayout.DEFAULT_SIZE,
                                                    GroupLayout.PREFERRED_SIZE)
                                            .add(3, 3, 3)));
            propPanelLayout
                    .setVerticalGroup(propPanelLayout
                            .createParallelGroup(GroupLayout.TRAILING)
                            .add(
                                    propPanelLayout
                                            .createSequentialGroup()
                                            .add(
                                                    propPanelLayout
                                                            .createParallelGroup(
                                                                    GroupLayout.LEADING)
                                                            .add(
                                                                    propPanelLayout
                                                                            .createSequentialGroup()
                                                                            .add(
                                                                                    propPanelLayout
                                                                                            .createParallelGroup(
                                                                                                    GroupLayout.BASELINE)
                                                                                            .add(
                                                                                                    scale)
                                                                                            .add(
                                                                                                    scaleTextFild,
                                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                                    GroupLayout.DEFAULT_SIZE,
                                                                                                    GroupLayout.PREFERRED_SIZE)
                                                                                            .add(
                                                                                                    fitToPageCheckBox)
                                                                                            .add(
                                                                                                    orientation))
                                                                            .add(
                                                                                    20,
                                                                                    20,
                                                                                    20)
                                                                            .add(
                                                                                    printBgroundChBox,
                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                    15,
                                                                                    GroupLayout.PREFERRED_SIZE))
                                                            .add(
                                                                    portraitPanel,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    Short.MAX_VALUE)
                                                            .add(
                                                                    landscapePanel,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    Short.MAX_VALUE))
                                            .addContainerGap()));
            propPanel.setLayout(propPanelLayout);
        }

        if (Xpcom.isWindows())
        {
            GroupLayout propPanelLayout = new GroupLayout(propPanel);
            propPanel.setLayout(propPanelLayout);
            propPanelLayout
                    .setHorizontalGroup(propPanelLayout
                            .createParallelGroup(GroupLayout.LEADING)
                            .add(
                                    propPanelLayout
                                            .createSequentialGroup()
                                            .add(
                                                    propPanelLayout
                                                            .createParallelGroup(
                                                                    GroupLayout.LEADING)
                                                            .add(
                                                                    propPanelLayout
                                                                            .createSequentialGroup()
                                                                            .add(
                                                                                    scale)
                                                                            .addPreferredGap(
                                                                                    LayoutStyle.RELATED)
                                                                            .add(
                                                                                    scaleTextFild,
                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                    74,
                                                                                    GroupLayout.PREFERRED_SIZE)
                                                                            .add(
                                                                                    9,
                                                                                    9,
                                                                                    9)
                                                                            .add(
                                                                                    fitToPageCheckBox)
                                                                            .add(
                                                                                    30,
                                                                                    30,
                                                                                    30)
                                                                            .add(
                                                                                    orientation))
                                                            .add(
                                                                    printBgroundChBox))
                                            .add(portraitPanel, 5, 55,
                                                    GroupLayout.PREFERRED_SIZE)
                                            .add(landscapePanel,
                                                    GroupLayout.PREFERRED_SIZE,
                                                    55,
                                                    GroupLayout.PREFERRED_SIZE)
                                            .addContainerGap()));
            propPanelLayout
                    .setVerticalGroup(propPanelLayout
                            .createParallelGroup(GroupLayout.TRAILING)
                            .add(
                                    propPanelLayout
                                            .createSequentialGroup()
                                            .add(
                                                    propPanelLayout
                                                            .createParallelGroup(
                                                                    GroupLayout.LEADING)
                                                            .add(
                                                                    landscapePanel,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    72,
                                                                    Short.MAX_VALUE)
                                                            .add(
                                                                    portraitPanel,
                                                                    GroupLayout.DEFAULT_SIZE,
                                                                    72,
                                                                    Short.MAX_VALUE)
                                                            .add(
                                                                    propPanelLayout
                                                                            .createSequentialGroup()
                                                                            .add(
                                                                                    propPanelLayout
                                                                                            .createParallelGroup(
                                                                                                    GroupLayout.BASELINE)
                                                                                            .add(
                                                                                                    scale)
                                                                                            .add(
                                                                                                    scaleTextFild,
                                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                                    GroupLayout.DEFAULT_SIZE,
                                                                                                    GroupLayout.PREFERRED_SIZE)
                                                                                            .add(
                                                                                                    fitToPageCheckBox)
                                                                                            .add(
                                                                                                    orientation))
                                                                            .add(
                                                                                    20,
                                                                                    20,
                                                                                    20)
                                                                            .add(
                                                                                    printBgroundChBox,
                                                                                    GroupLayout.PREFERRED_SIZE,
                                                                                    15,
                                                                                    GroupLayout.PREFERRED_SIZE)))
                                            .add(20, 20, 20)));
            propPanel.setLayout(propPanelLayout);
        }

        return propPanel;
    }

    JPanel createFramesPane()
    {
        JPanel framesPanel = new JPanel();
        framesPanel.setBorder(new CompoundBorder(new EmptyBorder(12, 0, 0, 0),
                new TitledBorder("Print frames")));
        framesPanel.setLayout(new AbsoluteLayout());
        oneFrameOnePageRButton = new JRadioButton();
        oneFrameOnePageRButton.addKeyListener(closeKeyAdapter);

        oneFrameOnePageRButton.setIcon(new ImageIcon(getClass().getResource(
                "res/per_page.png")));
        oneFrameOnePageRButton.setSelectedIcon(new ImageIcon(getClass()
                .getResource("res/per_page_sel.png")));

        ButtonGroup buttonGroup = new ButtonGroup();
        buttonGroup.add(oneFrameOnePageRButton);

        JPanel cell0 = new JPanel();
        cell0.setPreferredSize(new Dimension(70, 70));
        cell0.setLayout(new BoxLayout(cell0, BoxLayout.Y_AXIS));

        asIsRButton = new JRadioButton();
        asIsRButton.addKeyListener(closeKeyAdapter);

        asIsRButton.setIcon(new ImageIcon(getClass()
                .getResource("res/asis.png")));
        asIsRButton.setSelectedIcon(new ImageIcon(getClass().getResource(
                "res/asissel.png")));
        asIsRButton.setSelected(true);
        buttonGroup.add(asIsRButton);
        asIsRButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell0.add(asIsRButton);
        JLabel asIsCaption = new JLabel("As laid out");
        asIsCaption.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell0.add(asIsCaption);
        if (Xpcom.isWindows())
        {
            framesPanel.add(cell0, new AbsoluteConstraints(3, 30, 74, 89));
        }
        else if (Xpcom.isLinux())
        {
            framesPanel.add(cell0, new AbsoluteConstraints(10, 30, 74, 82));
        }
        else if (Xpcom.isMacOSX())
        {
            framesPanel.add(cell0, new AbsoluteConstraints(5, 34, 76, 82));
        }

        JPanel cell1 = new JPanel();
        cell1.setLayout(new BoxLayout(cell1, BoxLayout.Y_AXIS));
        onlySelectedRButton = new JRadioButton();
        onlySelectedRButton.addKeyListener(closeKeyAdapter);

        onlySelectedRButton.setIcon(new ImageIcon(getClass().getResource(
                "res/onlyselected.png")));
        onlySelectedRButton.setSelectedIcon(new ImageIcon(getClass()
                .getResource("res/onlyselectedsel.png")));
        buttonGroup.add(onlySelectedRButton);
        onlySelectedRButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell1.add(onlySelectedRButton);
        JLabel onlySelected = new JLabel("Selected frame");
        onlySelected.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell1.add(onlySelected);
        if (Xpcom.isWindows())
        {
            framesPanel.add(cell1, new AbsoluteConstraints(80, 30, 85, 89));
        }
        else if (Xpcom.isLinux())
        {
            framesPanel.add(cell1, new AbsoluteConstraints(98, 30, 95, 82));
        }
        else if (Xpcom.isMacOSX())
        {
            framesPanel.add(cell1, new AbsoluteConstraints(105, 34, 105, 82));
        }
        JPanel cell2 = new JPanel();
        cell2.setLayout(new BoxLayout(cell2, BoxLayout.Y_AXIS));
        oneFrameOnePageRButton.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell2.add(oneFrameOnePageRButton);
        if (Xpcom.isWindows())
        {
            framesPanel.add(cell2, new AbsoluteConstraints(158, 30, 92, 89));
        }
        else if (Xpcom.isLinux())
        {
            framesPanel.add(cell2, new AbsoluteConstraints(193, 30, 100, 82));
        }
        else if (Xpcom.isMacOSX())
        {
            framesPanel.add(cell2, new AbsoluteConstraints(205, 34, 130, 82));
        }

        JLabel oneFrameOnePage = new JLabel("Frame per page");
        oneFrameOnePage.setAlignmentX(Component.CENTER_ALIGNMENT);
        cell2.add(oneFrameOnePage);
        return framesPanel;
    }

    JPanel createConfirmButtonsPane()
    {
        // Add the print and cancel buttons
        JPanel buttonsPane = new JPanel();
        buttonsPane.setBorder(new EmptyBorder(7, 7, 7, 7));
        buttonsPane.setLayout(new FlowLayout(FlowLayout.RIGHT, 7, 7));

        JButton printBtn = new JButton(new PrintAction());
        JButton cancelButton = new JButton(new CancelAction());
        if (!Xpcom.isMacOSX())
        {
            printBtn.setPreferredSize(new Dimension(76, 25));
            cancelButton.setPreferredSize(new Dimension(76, 25));
        }
        else
        {
            printBtn.setPreferredSize(new Dimension(100, 25));
            cancelButton.setPreferredSize(new Dimension(100, 25));
        }

        buttonsPane.add(printBtn);
        buttonsPane.add(cancelButton);
        return buttonsPane;
    }

    private void initializePrinters()
    {
        Vector<String> printerNames = new Vector<String>();
        try
        {
            printerNames.addAll(_printingAPI.getAvailablePrintersNames());
        }
        catch (Exception e)
        {
        }
        DefaultComboBoxModel printerComboBoxModel = new DefaultComboBoxModel(
                printerNames);
        printerComboBox.setModel(printerComboBoxModel);
        noPrinter = printerNames.isEmpty();
    }

    private SaveJFileChooser fileChooser;
    private PrintToFileSettings getPrintToFileSettings()
    {
        final PrintToFileSettings printToFileSettings = new PrintToFileSettings();

        fileChooser = new SaveJFileChooser(this, new Runnable(){
            public void run() {

                String extension = null;
                if (!fileChooser.getFileFilter().getDescription().equals("All Files")) {
                    extension = ((FileNameExtensionFilter) fileChooser.getFileFilter()).getExtensions()[0];

                    if (extension.equals("pdf")) {
                        printToFileSettings.setOutputFormat(PrintToFileSettings.Format.PDF);
                    }
                    if (extension.equals("ps")) {
                        printToFileSettings.setOutputFormat(PrintToFileSettings.Format.PostScript);
                    }
                }
                else {
                    printToFileSettings.setOutputFormat(PrintToFileSettings.Format.NativePrintenter);
                }

                File selectedFile = fileChooser.getSelectedFile();
                File file;
                if (extension != null && -1 == selectedFile.getName().lastIndexOf(extension)) {
                    file = new File(selectedFile.getAbsolutePath() + "." + extension);
                } else {
                    file = selectedFile;
                }
                printToFileSettings.setFile(file);
            }
        }, true);

        //fileChooser.remove
        fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("ps", "Post Script files"));
        fileChooser.addChoosableFileFilter(new FileNameExtensionFilter("pdf", "PDF files"));
        
        fileChooser.setVisible(true);
        if (JFileChooser.CANCEL_OPTION == fileChooser.getChoose()) {
            return null;
        } else {
            return printToFileSettings;
        }
    }

    @SuppressWarnings( { "unused", "serial" })
    private class PropertiesAction extends AbstractAction
    {
        PropertiesAction()
        {
            super("Preferences...");
        }

        public void actionPerformed(ActionEvent e)
        {
        }
    }

    private static class MyPrintProgressListener implements PrintProgressListener
    {
        private String title;
        ProgressDialog progressDialog;

        MyPrintProgressListener(final String title)
        {
            this.title = title;
            progressDialog = new ProgressDialog(null, title);
        }

        public void printingStarted(PrintSettings printSettings)
        {
            try
            {
                SwingUtilities.invokeLater(new Runnable()
                {
                    public void run()
                    {
                       progressDialog.setVisible(true);
                    }
                });
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }

        public void printingFinished()
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    progressDialog.dispose();
                }
            });
        }

        public void progressChanged(final int curProgress, final int maxProgress)
        {
            SwingUtilities.invokeLater(new Runnable()
            {
                public void run()
                {
                    progressDialog.getProgressBar().setMaximum(maxProgress);
                    progressDialog.getProgressBar().setValue(curProgress);
                }
            });
        }
    }

    @SuppressWarnings("serial")
    private class PrintAction extends AbstractAction
    {
        PrintAction()
        {
            super("Print");
        }

        public void actionPerformed(ActionEvent e)
        {
            PrintSettings settings = new PrintSettings();

            settings.setPrinterName((String) printerComboBox.getSelectedItem());
   
            settings.setShrinkToFit(fitToPageCheckBox.isSelected());

            // If print to file
            if (printToFileChBox.isSelected())
            {
                settings.setPrinterName("");    
                PrintToFileSettings printToFileSettings = getPrintToFileSettings();
                if (null != printToFileSettings)
                {
                    settings.setPrintToFileSettings(printToFileSettings);
                }
                else
                {
                    return;
                }
            }
            // Copyes
            settings.setNumCopies((Integer) numberOfCopiesSpinner.getValue());

            // Range
            if (allPagesRadioButton.isSelected())
            {
                settings.setRange(new PageRange(RangeType.All));
            }
            else
            {
                PageRange range = new PageRange(Integer.valueOf(firstPage
                        .getText()), lastPage.getValue());
                settings.setRange(range);
            }

            // Frames
            if (asIsRButton.isSelected())
            {
                settings.setFrameSettings(new FrameSettings(
                        HowToEnableFrameUI.EnableAsIsAndEach,
                        PrintFrameTypeUsage.AsIs));
            }
            else if (onlySelectedRButton.isSelected())
            {
                settings.setFrameSettings(new FrameSettings(
                        HowToEnableFrameUI.EnableAll,
                        PrintFrameTypeUsage.SelectedFrame));
            }
            else if (oneFrameOnePageRButton.isSelected())
            {
                settings.setFrameSettings(new FrameSettings(
                        HowToEnableFrameUI.EnableAsIsAndEach,
                        PrintFrameTypeUsage.EachFrameSeparated));
            }

            // Scale
            if (!fitToPageCheckBox.isSelected())
            {
                settings.setScaling(scaleTextFild.getScaleValue());
            }

            // Orinetation
            if (portraitOrRButton.isSelected())
            {
                settings.setOrientation(PageOrientation.Portrait);
            }
            else
            {
                settings.setOrientation(PageOrientation.Landscape);
            }

            // Backgrounds
            if (printBgroundChBox.isSelected())
            {
                settings.setPrintBackgroundColors(true);
                settings.setPrintBackgroundImages(true);
            }

            _printingAPI.initPrintSettings(settings);
            final String title = settings.getPrinterName() + "printing prgogress";
            final MyPrintProgressListener prgListener = new MyPrintProgressListener(title);
            _printingAPI.print(prgListener);
            //_printingAPI.print(null);
            dispose();
        }
    }

    @SuppressWarnings("serial")
    private class CancelAction extends AbstractAction
    {

        public CancelAction()
        {
            super("Cancel");
        }

        public void actionPerformed(ActionEvent e)
        {
            dispose();
        }

    }

    private class NumberTextFild extends JTextField
    {
        private static final long serialVersionUID = -4070443375908154900L;

        NumberTextFild()
        {
            super();
            if (!Xpcom.isLinux() || !Xpcom.isMacOSX())
            {
                setPreferredSize(new Dimension(37, 22));
            }
            else
            {
                setPreferredSize(new Dimension(35, 22));
            }
            addKeyListener(closeKeyAdapter);
            addKeyListener(new KeyListener()
            {
                public void keyPressed(KeyEvent e)
                {
                }

                public void keyReleased(KeyEvent e)
                {
                }

                public void keyTyped(KeyEvent e)
                {
                    if (e.getKeyChar() < '0' || e.getKeyChar() > '9')
                    {
                        e.setKeyChar((char) KeyEvent.VK_BACK_SPACE);
                    }
                }
            });

            addFocusListener(new FocusListener()
            {

                public void focusGained(FocusEvent e)
                {
                }

                public void focusLost(FocusEvent e)
                {
                    try
                    {
                        Integer _int = Integer.valueOf(getText());
                        if (0 == _int)
                        {
                            setText("1");
                        }
                    }
                    catch (NumberFormatException ex)
                    {
                        setText("1");
                    }
                }
            });
        }

        int getValue()
        {
            if (getText() == null || getText().equals(""))
            {
                return 0;
            }
            return Integer.valueOf(getText());
        }

        void setValue(int value)
        {
            setText(String.valueOf(value));
        }
    }

    private class ScaleTextFild extends JFormattedTextField/* JTextField */
    {
        private static final long serialVersionUID = -471137136991200333L;
        final private Color bgColorEnabled = new Color(255, 255, 255);
        final private Color bgColorDisable;

        public ScaleTextFild()
        {
            super(new InternationalFormatter()
            {
                private final DocumentFilter filter = new PercentFilter();

                @Override
                protected DocumentFilter getDocumentFilter()
                {
                    return filter;
                }
            });
            setInputVerifier(new ScaleVerifier());
            setText("100%");

            if (Xpcom.isMacOSX() || Xpcom.isLinux())
            {
                bgColorDisable = new JTextField().getBackground();
            }
            else
            {
                bgColorDisable = new Color(212, 208, 200);
            }
        }

        @Override
        public void setEnabled(boolean enabled)
        {
            super.setEnabled(enabled);
            if (enabled)
            {
                setBackground(bgColorEnabled);
            }
            else
            {
                setBackground(bgColorDisable);
            }
        }

        public Double getScaleValue()
        {
            Double value = Double.valueOf(getText().substring(0,
                    getText().length() - 1));
            return (value / 100);
        }

        public void setScaleValue(Double scaleValuePercent)
        {
            setText(scaleValuePercent + "%");
        }
    }

    private class ScaleVerifier extends InputVerifier
    {
        private final int stringLenth = 3;

        @Override
        public boolean verify(JComponent component)
        {
            JFormattedTextField field = (JFormattedTextField) component;
            String text = field.getText();
            if (text != null && !text.equals(""))
            {
                if (text.endsWith("%"))
                {
                    text = text.substring(0, text.length() - 1);
                }
                if (text.length() > stringLenth)
                {
                    field.setText(text.substring(0, stringLenth));
                }
            }
            else
            {
                text = "100";
            }
            try
            {
                if (Double.parseDouble(text) >= 100D
                        || Double.parseDouble(text) < 1D)
                {
                    text = "100";
                }
            }
            catch (NumberFormatException e1)
            {
                text = "100";
            }
            field.setText(text + "%");

            return field.isEditValid();
        }
    }

    private class PercentFilter extends DocumentFilter
    {
        @Override
        public void insertString(FilterBypass fb, int offset, String string,
                AttributeSet attr) throws BadLocationException
        {
            StringBuilder builder = filtering(string);
            super.insertString(fb, offset, builder.toString(), attr);
        }

        @Override
        public void replace(FilterBypass fb, int offset, int length,
                String string, AttributeSet attr) throws BadLocationException
        {
            if (string != null)
            {
                StringBuilder builder = filtering(string);
                string = builder.toString();
            }
            super.replace(fb, offset, length, string, attr);
        }

        private StringBuilder filtering(String string)
        {
            StringBuilder builder = new StringBuilder(string);
            for (int i = builder.length() - 1; i >= 0; i--)
            {
                int cp = builder.codePointAt(i);
                if (!(Character.isDigit(cp) || cp == 37 || cp == 46)) // 37 - %
                {
                    builder.deleteCharAt(i);
                    if (Character.isSupplementaryCodePoint(cp))
                    {
                        i--;
                        builder.deleteCharAt(i);
                    }
                }
            }
            return builder;
        }
    }

    @SuppressWarnings("unused")
    private static String TYPE_UNKNOWN = "Type Unknown";

    @SuppressWarnings("unused")
    private static String HIDDEN_FILE = "Hidden File";

    private class FileNameExtensionFilter extends FileFilter
    {

        private Map<String, FileNameExtensionFilter> filters = null;

        private String description = null;

        private String fullDescription = null;

        private boolean useExtensionsInDescription = true;

        public FileNameExtensionFilter()
        {
            this.filters = new HashMap<String, FileNameExtensionFilter>();
        }

        public FileNameExtensionFilter(String extension)
        {
            this(extension, null);
        }

        public FileNameExtensionFilter(String extension, String description)
        {
            this();
            if (extension != null) addExtension(extension);
            if (description != null) setDescription(description);
        }

        public FileNameExtensionFilter(String[] filters)
        {
            this(filters, null);
        }

        public FileNameExtensionFilter(String[] filters, String description)
        {
            this();
            for (String filter : filters)
            {
                // add filters one by one
                addExtension(filter);
            }
            if (description != null) setDescription(description);
        }

        @Override
        public boolean accept(File f)
        {
            if (f != null)
            {
                if (f.isDirectory())
                {
                    return true;
                }
                String extension = getExtension(f);
                if (extension != null && filters.get(getExtension(f)) != null)
                {
                    return true;
                }
            }
            return false;
        }

        public String getExtension(File f)
        {
            if (f != null)
            {
                String filename = f.getName();
                int i = filename.lastIndexOf('.');
                if (i > 0 && i < filename.length() - 1)
                {
                    return filename.substring(i + 1).toLowerCase();
                }
            }
            return null;
        }

        public void addExtension(String extension)
        {
            if (filters == null)
            {
                filters = new HashMap<String, FileNameExtensionFilter>(5);
            }
            filters.put(extension.toLowerCase(), this);
            fullDescription = null;
        }

        @Override
        public String getDescription()
        {
            if (fullDescription == null)
            {
                if (description == null || isExtensionListInDescription())
                {
                    fullDescription = description == null ? "(" : description
                            + " (";
                    // build the description from the extension list
                    Set<String> extensions = filters.keySet();
                    if (extensions != null)
                    {
                        Iterator<String> it = extensions.iterator();
                        fullDescription += "*." + it.next();
                        while (it.hasNext())
                        {
                            fullDescription += ", *." + it.next();
                        }
                    }
                    fullDescription += ")";
                }
                else
                {
                    fullDescription = description;
                }
            }
            return fullDescription;
        }

        String[] getExtensions()
        {
            String ext[] = new String[filters.size()];
            int i = 0;
            for (String str : filters.keySet())
            {
                String[] split = str.split(" ");
                ext[i] = split[0];
                i++;
            }
            return ext;
        }

        public void setDescription(String description)
        {
            this.description = description;
            fullDescription = null;
        }

        public void setExtensionListInDescription(boolean b)
        {
            useExtensionsInDescription = b;
            fullDescription = null;
        }

        public boolean isExtensionListInDescription()
        {
            return useExtensionsInDescription;
        }
    }

    private class CloseKeyAdapter extends KeyAdapter
    {
        @Override
        public void keyPressed(KeyEvent e)
        {
            if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
            {
                dispose();
            }
        }
    }

    public void setPageTo(int pageToNumber)
    {
        lastPage.setValue(pageToNumber);
    }

    public void setPortraitButton(boolean stateSelectedButton)
    {
        portraitOrRButton.setSelected(stateSelectedButton);
    }

    public void setLandscapeButton(boolean stateSelectedButton)
    {
        landscapeOrRButton.setSelected(stateSelectedButton);
    }

    public void setBackgroundChBox(boolean stateSelectedButton)
    {
        printBgroundChBox.setSelected(stateSelectedButton);
    }
}
