/*
 * Copyright (c) 1998-2008 TeamDev Ltd. All Rights Reserved.
 * Use is subject to license terms.
 */
package com.teamdev.xpcom.mozilla.demo.googlemaps; 

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Vector;
import java.util.concurrent.atomic.AtomicBoolean;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JRadioButton;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;

import com.teamdev.jxbrowser.WebBrowser;
import com.teamdev.jxbrowser.WebBrowserFactory;
import com.teamdev.jxbrowser.event.ProgressEvent;
import com.teamdev.jxbrowser.event.RequestAdapter;
import com.teamdev.jxbrowser.event.StateEvent;
import com.teamdev.jxbrowser.event.StatusEvent;
import com.teamdev.xpcom.Xpcom;

/**
 * JxBrowser for google maps service demo program. (Search engine for travelers)
 * 
 * @author Victor Gubin, TeamDev ltd.
 * 
 */
public class GoogleMapsDemo extends JFrame {

	private static final String GOOGLE_MAPS = "http://maps.google.com/maps?f=l";

	private static final String TEAMDEV_LOC = "http://maps.google.com/maps?f=l&hl=en"
			+ "&geocode=&q=TeamDev&near=Kharkov&ie=UTF8"
			+ "&ll=50.038496,36.221141&spn=0.004817,0.0" + "09398&t=h&z=17";

	private static final char JSC_DEL = '"';

	JComboBox countryCB;
	JComboBox cityCB;
	JRadioButton hotel;
	JRadioButton pizza;
	JRadioButton gaz;
	JRadioButton airport;
	JButton teamdevLocation;
	JProgressBar progressBar;
	JLabel statusStr;

	WebBrowser browser;

	DefaultComboBoxModel austriaCBM;
	DefaultComboBoxModel germanyCBM;
	DefaultComboBoxModel switzerlandCBM;
	DefaultComboBoxModel ukraineCBM;
	DefaultComboBoxModel usaCBM;
	DefaultComboBoxModel indiaCBM;
	DefaultComboBoxModel italyCBM;
	DefaultComboBoxModel russiaCBM;

	final AtomicBoolean mutex = new AtomicBoolean(false);

	private class BrowserRequestListener extends RequestAdapter {

		@Override
		public void progressChanged(final ProgressEvent e) {
			SwingUtilities.invokeLater(new Runnable() {

				public void run() {
					progressBar.setMaximum((int) e.getMaxTotalProgress());
					progressBar.setValue((int) e.getCurTotalProgress());
				}

			});
		}

		@Override
		public void stateChanged(StateEvent e) {
			if (!e.isLoadingDocument() && e.isNetwork()) {
				synchronized (mutex) {
					mutex.set(true);
					mutex.notifyAll();
				}
			}
		}

		@Override
		public void statusChanged(final StatusEvent e) {
			SwingUtilities.invokeLater(new Runnable() {
				public void run() {
					statusStr.setText(e.getMessage());
				}
			});
		}

	}

	GoogleMapsDemo() {
		super();
		browser = WebBrowserFactory.getInstance().createBrowser();
		browser.navigate(GOOGLE_MAPS);

		countryCB = new JComboBox();
		Vector<String> countries = new Vector<String>();
		countries.add("Austria");
		countries.add("Germany");
		countries.add("Switzerland");
		countries.add("Ukraine");
		countries.add("USA");
		countries.add("Indea");
		countries.add("Italy");
		countries.add("Russia");
		countryCB.setModel(new DefaultComboBoxModel(countries));

		JComboBox city = new JComboBox();
		Vector<String> citiesAustria = new Vector<String>();
		citiesAustria.add("Wien");
		citiesAustria.add("Graz");
		austriaCBM = new DefaultComboBoxModel(citiesAustria);

		Vector<String> citiesGermany = new Vector<String>();
		citiesGermany.add("Berlin");
		citiesGermany.add("Munchen");
		germanyCBM = new DefaultComboBoxModel(citiesGermany);

		Vector<String> citiesSwitzerland = new Vector<String>();
		citiesSwitzerland.add("Bern");
		citiesSwitzerland.add("Zurich");
		switzerlandCBM = new DefaultComboBoxModel(citiesSwitzerland);

		Vector<String> citiesUkraine = new Vector<String>();
		citiesUkraine.add("Kiev");
		citiesUkraine.add("Kharkov");
		ukraineCBM = new DefaultComboBoxModel(citiesUkraine);

		Vector<String> citiesUSA = new Vector<String>();
		citiesUSA.add("Washington city");
		citiesUSA.add("New-York city");
		usaCBM = new DefaultComboBoxModel(citiesUSA);

		Vector<String> citiesIndea = new Vector<String>();
		citiesIndea.add("New Delhi");
		citiesIndea.add("Mumbai"); // Bombay
		indiaCBM = new DefaultComboBoxModel(citiesIndea);

		Vector<String> citiesItaly = new Vector<String>();
		citiesItaly.add("Roma");
		citiesItaly.add("Milano");
		italyCBM = new DefaultComboBoxModel(citiesItaly);

		Vector<String> citiesRussia = new Vector<String>();
		citiesRussia.add("Moscow");
		citiesRussia.add("St. Peterburg");
		russiaCBM = new DefaultComboBoxModel(citiesRussia);

		cityCB = new JComboBox();
		cityCB.setModel(austriaCBM);

		browser.addRequestListener(new BrowserRequestListener());
		browser.navigate(GOOGLE_MAPS);
		waitUtilDone();

		hotel = new JRadioButton("Hotels");
		pizza = new JRadioButton("Pizza");
		gaz = new JRadioButton("Gas stations");
		airport = new JRadioButton("Airports");

		ButtonGroup group = new ButtonGroup();
		group.add(hotel);
		group.add(pizza);
		group.add(gaz);
		group.add(airport);

		teamdevLocation = new JButton("TeamDev ltd.");

		progressBar = new JProgressBar();
		statusStr = new JLabel();
	}

	void initBL() {
		countryCB.addItemListener(new ItemListener() {

			public void itemStateChanged(ItemEvent e) {

				switch (countryCB.getSelectedIndex()) {
				case 0:
					cityCB.setModel(austriaCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 1:
					cityCB.setModel(germanyCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 2:
					cityCB.setModel(switzerlandCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 3:
					cityCB.setModel(ukraineCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 4:
					cityCB.setModel(usaCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 5:
					cityCB.setModel(indiaCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 6:
					cityCB.setModel(italyCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
					break;
				case 7:
					cityCB.setModel(russiaCBM);
					fillWhereFild(countryCB.getSelectedItem(), cityCB
							.getSelectedItem());
				}
			}
		});

		cityCB.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				fillWhereFild(countryCB.getSelectedItem(), cityCB
						.getSelectedItem());
			}
		});

		hotel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				fillWhatFild("Hotel");
			}
		});

		pizza.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fillWhatFild("Pizza");
			}

		});

		gaz.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fillWhatFild("Gas station");
			}
		});

		airport.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				fillWhatFild("Airport");
			}

		});

		teamdevLocation.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				browser.navigate(TEAMDEV_LOC);
				waitUtilDone();
			}
		});
	}

	private void fillWhereFild(Object country, Object city) {
		browser.evaluateScript("document.forms[1].near.value=" + JSC_DEL
				+ (String) country + ", " + (String) city + JSC_DEL);
	}

	private void fillWhatFild(Object what) {
		browser.evaluateScript("document.forms[1].q.value=" + JSC_DEL
				+ (String) what + JSC_DEL);
		browser.evaluateScript("document.forms[1].btnG.click();");
		waitUtilDone();
	}

	private void waitUtilDone() {
		synchronized (mutex) {
			if (!mutex.get()) {
				try {
					mutex.wait(1000);
				} catch (InterruptedException e) {
				}
			}
		}
		mutex.set(false);
	}

	void initUI() {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout());

		JPanel statusPanel = new JPanel();
		statusPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 5, 5));
		statusPanel.add(progressBar);
		statusPanel.add(statusStr);
		add(statusPanel, BorderLayout.SOUTH);

		JPanel toolPanel = new JPanel();
		toolPanel.setLayout(new BorderLayout());
		toolPanel.setBorder(new EmptyBorder(9, 9, 9, 9));

		JPanel wherePanel = new JPanel();
		wherePanel.setLayout(new GridLayout(2, 2));
		wherePanel.setBorder(new TitledBorder("Where"));
		wherePanel.add(new JLabel(" Country"));
		wherePanel.add(countryCB);
		wherePanel.add(new JLabel(" City"));
		wherePanel.add(cityCB);
		toolPanel.add(wherePanel, BorderLayout.NORTH);

		JPanel whatPanel = new JPanel();
		whatPanel.setBorder(new TitledBorder("What"));
		whatPanel.setLayout(new BoxLayout(whatPanel, BoxLayout.PAGE_AXIS));

		whatPanel.add(hotel);
		whatPanel.add(pizza);
		whatPanel.add(gaz);
		whatPanel.add(airport);
		toolPanel.add(whatPanel, BorderLayout.CENTER);

		toolPanel.add(teamdevLocation, BorderLayout.AFTER_LAST_LINE);

		add(toolPanel, BorderLayout.WEST);
		add(browser.getComponent(), BorderLayout.CENTER);
		setLocationByPlatform(false);
		setTitle("JxBrowser demo for Google Maps");
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		setSize(dim.width, dim.height - 27);
		setVisible(true);
	}

	public static void main(String args[]) {
		Xpcom.initialize(Xpcom.AWT);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				GoogleMapsDemo sample = new GoogleMapsDemo();
				sample.initBL();
				sample.initUI();
			}
		});
	}
}
